#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for reading in polynomial data from the csv files.

@author: mark
"""

import logging
import numpy as np
import csv

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

def parse_polynomial_data_file_list(filename_list, **kwargs):
    """This parses each file, and then returns the dictionary containing the
    information, by solvent ID.
    """
    polynomial_data_by_solventid ={}
    for filename in filename_list:
        solvent_id = generate_solvent_id(filename, kwargs.get("suffix", ".csv"))
        polynomial_fit_data = parse_polynomial_data_file(filename)
        polynomial_data_by_solventid[solvent_id] = polynomial_fit_data
    return polynomial_data_by_solventid

def generate_solvent_id(filename, suffix):
    """This gets the solvent ID from the filename. assumes file name is of the
    form '*/solventID.csv' where 'solventID' is the solvent ID.
    """
    return filename.split('/')[-1].replace(suffix, '').replace("_", ",")

def parse_polynomial_data_file(filename):
    """This reads in the file and parses the polynomial lines.
    """
    polynomial_data_dict = {}
    data_in = read_polynomial_data_file(filename)
    for line_list in data_in[1:]:
        polynomial_data = read_polynomial_data_line(line_list)
        if "fit_range" in polynomial_data:
            fit_range = polynomial_data.pop("fit_range")
            if polynomial_data["order"] in polynomial_data_dict:
                polynomial_data_dict[polynomial_data["order"]][fit_range] = polynomial_data
            else:
                polynomial_data_dict[polynomial_data["order"]] = {fit_range:polynomial_data}
        else:
            polynomial_data_dict[polynomial_data["order"]] = polynomial_data
    return polynomial_data_dict

def read_polynomial_data_file(filename):
    """This reads in the tab separated variable file containing the polynomial fit data.
    """
    data_in = []
    with open(filename, "r") as file_in:
        file_contents = csv.reader(file_in, delimiter='\t')
        for line in file_contents:
            data_in.append(line)
    return data_in

def read_polynomial_data_line(line_list):
    """This reads in the values to a dictionary.
    This assumes that the values in the line are formatted such that they are
    ordered:
        Order, RMSE, covariance, coefficients.
    """
    order = int(line_list[0])
    fit_range = line_list[1]
    RMSE = float(line_list[2])
    covariance = float(line_list[3])
    coefficients = np.array([float(x) for x in line_list[4:]])
    if fit_range == "All":
        return {"order": order, "RMSE":RMSE, "covar":covariance,
                "coefficients":coefficients}
    else:
        return {"order": order, "RMSE":RMSE, "covar":covariance,
                "coefficients":coefficients, "fit_range":fit_range}
