#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for testing the linkagewriter script.

@author: mark
"""

import logging
import numpy as np
import os
import unittest
import solventmapcreator.io.linkagewriter as linkagewriter

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

class LinkageWriterTestCase(unittest.TestCase):
    """Test case for the linkagewriter script.
    """
    def setUp(self):
        """Set up before tests.
        """
        self.linkage_matrix = np.array([[0., 1., 17., 2.],
                                        [4., 5., 22., 3.],
                                        [2., 3., 28., 2.],
                                        [6., 7., 33., 5.]])
        self.expected_filename = "resources/expectedlinkagematrix.csv"
        self.inconsistency_matrix = np.array([[17., 0., 1., 0.],
                                              [19.5, 3.535534, 2., 0.707107],
                                              [28., 0., 1., 0.],
                                              [27.666667,5.507571, 3., 0.968364]])
        self.cophenetic_coefficient = 0.73020316123580109
        self.cophenetic_distance_matrix = np.array([17., 33., 33., 22., 33.,
                                                    33., 22., 28., 33., 33.])
        self.expected_cop_filename = "resources/expectedcopfile.csv"
    def tearDown(self):
        """Clean up after tests.
        """
        del self.linkage_matrix
        del self.expected_filename
        del self.inconsistency_matrix
        del self.cophenetic_coefficient
        del self.cophenetic_distance_matrix
        del self.expected_cop_filename
        if os.path.isfile("linkagematrix.csv"):
            os.remove("linkagematrix.csv")
        if os.path.isfile("coph_incons.csv"):
            os.remove("coph_incons.csv")
    def test_write_linkage_matrix(self):
        """Test to see if expected file is produced.
        """
        actual_filename = "linkagematrix.csv"
        link_mat_out = linkagewriter.write_linkage_matrix(self.linkage_matrix,
                                                          actual_filename)
        self.assertEqual(0, link_mat_out)
        with open(self.expected_filename, 'r') as exp_file:
            exp_file_lines = exp_file.readlines()
            with open(actual_filename, 'r') as act_file:
                act_file_lines = act_file.readlines()
                self.assertListEqual(act_file_lines, exp_file_lines)
    def test_create_linkage_file_line(self):
        """Test to see if line is written as expected.
        """
        expected_list = ['0', '1', '17.0000000', '2']
        actual_list = linkagewriter.create_linkage_file_line(self.linkage_matrix[0])
        self.assertListEqual(expected_list, actual_list)
    def test_write_cophenetic_and_inconsistency(self):
        """Test to see if expected file is produced
        """
        actual_filename = "coph_incons.csv"
        coph_out = linkagewriter.write_cophenetic_and_inconsistency(self.cophenetic_coefficient,
                                                                    self.cophenetic_distance_matrix,
                                                                    self.inconsistency_matrix,
                                                                    actual_filename)
        with open(self.expected_cop_filename, 'r') as exp_file:
            exp_file_lines = exp_file.readlines()
            with open(actual_filename, 'r') as act_file:
                act_file_lines = act_file.readlines()
                self.assertListEqual(act_file_lines, exp_file_lines)
    def test_create_cophenetic_lines(self):
        """Test to see if expected lines are produced.
        """
        expected_lines = [['Cophenetic coefficient', '0.7302032'],
                          ['Cophenetic Distance Matrix'],
                          ['0.0000000', '17.0000000', '33.0000000', '33.0000000', '22.0000000'],
                          ['17.0000000', '0.0000000', '33.0000000', '33.0000000', '22.0000000'],
                          ['33.0000000', '33.0000000', '0.0000000', '28.0000000', '33.0000000'],
                          ['33.0000000', '33.0000000', '28.0000000', '0.0000000', '33.0000000'],
                          ['22.0000000', '22.0000000', '33.0000000', '33.0000000', '0.0000000']]
        actual_lines = linkagewriter.create_cophenetic_lines(self.cophenetic_coefficient,
                                                             self.cophenetic_distance_matrix)
        self.assertListEqual(expected_lines, actual_lines)
    def test_create_cophenetic_distance_line(self):
        """Test to see if expected distance line is produced.
        """
        expected_line = ['0.0000000', '17.0000000', '33.0000000', '33.0000000', '22.0000000']
        actual_line = linkagewriter.create_cophenetic_distance_line(np.array([0.0, 17., 33., 33., 22.]))
        self.assertListEqual(expected_line, actual_line)
    def test_create_inconsistency_lines(self):
        """Test to see if expected list of lists is produced.
        """
        expected_lines = [['Mean Heights of links', 'SD for links',
                           'Number of links', 'Inconsistency coefficient'],
                          ['17.0000000', '0.0000000', '1.0', '0.0000000'],
                          ['19.5000000', '3.5355340', '2.0', '0.7071070'],
                          ['28.0000000', '0.0000000', '1.0', '0.0000000'],
                          ['27.6666670', '5.5075710', '3.0', '0.9683640']]
        actual_lines = linkagewriter.create_inconsistency_lines(self.inconsistency_matrix)
        self.assertListEqual(expected_lines, actual_lines)
    def test_create_inconsistency_line(self):
        """Test to see if expected list is produced.
        """
        expected_line = ['17.0000000', '0.0000000', '1.0', '0.0000000']
        actual_line = linkagewriter.create_inconsistency_line(self.inconsistency_matrix[0])
        self.assertListEqual(expected_line, actual_line)

