#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for testing the solvation energy extraction methods.

@author: mark
"""

import logging
import unittest
import os
from lxml import etree
import solventmapcreator.io.solvationenergyextraction as solvationenergyextraction

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)

class SolvationEnergyExtractionTestCase(unittest.TestCase):
    """Test case for solvationenergyextraction.
    """
    def setUp(self):
        """Set up for tests
        """
        self.maxDiff=None
        self.solvent_filename = "resources/watersolvent.xml"
        self.solvent_xml = solvationenergyextraction.read_xml_file(self.solvent_filename)
        self.binding_energy_filename = "resources/energyvaluestest.xml"
        self.free_energy_filename = "resources/energyvaluestest2.xml"
        self.actual_output_filename = "resources/watersolvation.xml"
        self.actual_output_filename2 = "waterfree.xml"
        self.actual_output_filename3 = "waterbinding.xml"
        self.binding_energy_xml = solvationenergyextraction.read_xml_file(self.binding_energy_filename)
        self.free_energy_xml = solvationenergyextraction.read_xml_file(self.free_energy_filename)
    def tearDown(self):
        """clean up after tests.
        """
        del self.solvent_filename
        del self.binding_energy_filename
        del self.free_energy_filename
        if os.path.isfile(self.actual_output_filename):
            os.remove(self.actual_output_filename)
        del self.actual_output_filename
        if os.path.isfile(self.actual_output_filename2):
            os.remove(self.actual_output_filename2)
        del self.actual_output_filename2
        if os.path.isfile(self.actual_output_filename3):
            os.remove(self.actual_output_filename3)
        del self.actual_output_filename3
    def test_read_in_solvent_id_list(self):
        """Test to see if expected solvent ID list is returned.
        """
        expected_solvent_list = ["water"]
        actual_list = solvationenergyextraction.read_in_solvent_id_list(self.solvent_filename)
        self.assertListEqual(expected_solvent_list, actual_list)
    def test_get_solvent_id_list(self):
        """Test to see if expected solvent list is returned.
        """
        expected_solvent_list = ["water"]
        actual_list = solvationenergyextraction.get_solvent_id_list(self.solvent_xml)
        self.assertListEqual(expected_solvent_list, actual_list)
    def test_extract_and_write_energy_values_for_solvent_list(self):
        """Test to see if expected files is produced.
        """
        solvationenergyextraction.extract_and_write_energy_values_for_solvent_list(["water"], self.binding_energy_xml, "binding", "")
        with open(self.actual_output_filename3, "r") as actual_file:
            actual_xml = actual_file.read()
            with open("resources/energyvaluestest.xml") as expected_file:
                expected_xml = expected_file.read()
                self.assertEqual(expected_xml, actual_xml)
    def test_extract_and_write_energy_values_for_solvent(self):
        """Test to see if expected file is produced.
        """
        solvationenergyextraction.extract_and_write_energy_values_for_solvent("water", self.free_energy_xml, "free", "")
        with open(self.actual_output_filename2, "r") as actual_file:
            actual_xml = actual_file.read()
            with open("resources/energyvaluestest2.xml") as expected_file:
                expected_xml = expected_file.read()
                self.assertEqual(expected_xml, actual_xml)
    def test_generate_energy_values_element_for_solvent(self):
        """Test to see expected energy values is produced.
        """
        expected_xml = """<phase:EnergyValues xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema">
  <phase:FreeEnergyCollection><phase:FreeEnergy phase:moleculeID="-14.100solute" phase:fromSolventID="" phase:toSolventID="water" phase:valueType="MOLEFRACTION">
        <phase:TotalEnergy>-34.8195308084233</phase:TotalEnergy>
            <phase:EnergyContributions>
                <phase:EnergyContribution phase:ssipID="1">-34.8195308084233</phase:EnergyContribution>
            </phase:EnergyContributions>
        </phase:FreeEnergy>
  </phase:FreeEnergyCollection>
</phase:EnergyValues>
"""
        actual_element = solvationenergyextraction.generate_energy_values_element_for_solvent("water", self.free_energy_xml, "free")
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_element, pretty_print=True))
    def test_create_output_filename(self):
        """Test to see if expected filename is produced.
        """
        expected_filename = "waterfree.xml"
        actual_filename = solvationenergyextraction.create_output_filename("water", "free")
        self.assertEqual(expected_filename, actual_filename)
    def test_write_element_tree_to_file(self):
        """Test to see if expecetd file is written.
        """
        xpath_expression = solvationenergyextraction.generate_free_energy_xpath_expression("water")
        free_elements = solvationenergyextraction.get_energy_elements(self.free_energy_xml,
                                                                         xpath_expression)
        energy_values_element = solvationenergyextraction.generate_energy_values_element(free_elements, "free")
        energy_values_tree = solvationenergyextraction.create_element_tree(energy_values_element)
        solvationenergyextraction.write_element_tree_to_file(energy_values_tree, self.actual_output_filename)
        with open(self.actual_output_filename, "r") as actual_file:
            actual_xml = actual_file.read()
            with open("resources/energyvaluestest2.xml") as expected_file:
                expected_xml = expected_file.read()
                self.assertEqual(expected_xml, actual_xml)
    def test_create_element_tree(self):
        """Test to see if expected element tree is produced.
        """
        expected_xml = """<phase:EnergyValues xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema"/>
"""
        actual_element = solvationenergyextraction.create_element_tree(solvationenergyextraction.create_energy_values_element())
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_element, pretty_print=True))
    def test_generate_energy_values_element(self):
        """Test to see if expected element is produced.
        """
        expected_xml = """<phase:EnergyValues xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema">
  <phase:FreeEnergyCollection><phase:FreeEnergy phase:moleculeID="-14.100solute" phase:fromSolventID="" phase:toSolventID="water" phase:valueType="MOLEFRACTION">
        <phase:TotalEnergy>-34.8195308084233</phase:TotalEnergy>
            <phase:EnergyContributions>
                <phase:EnergyContribution phase:ssipID="1">-34.8195308084233</phase:EnergyContribution>
            </phase:EnergyContributions>
        </phase:FreeEnergy>
  </phase:FreeEnergyCollection>
</phase:EnergyValues>
"""
        xpath_expression = solvationenergyextraction.generate_free_energy_xpath_expression("water")
        free_elements = solvationenergyextraction.get_energy_elements(self.free_energy_xml,
                                                                         xpath_expression)
        actual_element = solvationenergyextraction.generate_energy_values_element(free_elements, "free")
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_element, pretty_print=True))
    def test_generate_free_energy_collection(self):
        """Test to see if expected element is produced.
        """
        expected_xml = """<phase:FreeEnergyCollection xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema"><phase:FreeEnergy phase:moleculeID="-14.100solute" phase:fromSolventID="" phase:toSolventID="water" phase:valueType="MOLEFRACTION">
        <phase:TotalEnergy>-34.8195308084233</phase:TotalEnergy>
            <phase:EnergyContributions>
                <phase:EnergyContribution phase:ssipID="1">-34.8195308084233</phase:EnergyContribution>
            </phase:EnergyContributions>
        </phase:FreeEnergy>
  </phase:FreeEnergyCollection>
"""
        xpath_expression = solvationenergyextraction.generate_free_energy_xpath_expression("water")
        free_elements = solvationenergyextraction.get_energy_elements(self.free_energy_xml,
                                                                         xpath_expression)
        actual_element = solvationenergyextraction.generate_free_energy_collection(free_elements)
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_element, pretty_print=True))
    def test_generate_binding_energy_collection(self):
        """Test to see if expected element is produced.
        """
        expected_xml = """<phase:BindingEnergyCollection xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema"><phase:BindingEnergy phase:moleculeID="-14.100solute" phase:fromSolventID="" phase:toSolventID="water" phase:valueType="MOLEFRACTION">
            <phase:TotalEnergy>-34.8195308084233</phase:TotalEnergy>
            <phase:EnergyContributions>
                <phase:EnergyContribution phase:ssipID="1">-34.8195308084233</phase:EnergyContribution>
            </phase:EnergyContributions>
        </phase:BindingEnergy>
  </phase:BindingEnergyCollection>
"""
        xpath_expression = solvationenergyextraction.generate_binding_energy_xpath_expression("water")
        binding_elements = solvationenergyextraction.get_energy_elements(self.binding_energy_xml,
                                                                        xpath_expression)
        actual_element = solvationenergyextraction.generate_binding_energy_collection(binding_elements)
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_element, pretty_print=True))
    def test_append_energies_to_collection(self):
        """Test to see if expected element is produced.
        """
        expected_xml = """<phase:BindingEnergyCollection xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema"><phase:BindingEnergy phase:moleculeID="-14.100solute" phase:fromSolventID="" phase:toSolventID="water" phase:valueType="MOLEFRACTION">
            <phase:TotalEnergy>-34.8195308084233</phase:TotalEnergy>
            <phase:EnergyContributions>
                <phase:EnergyContribution phase:ssipID="1">-34.8195308084233</phase:EnergyContribution>
            </phase:EnergyContributions>
        </phase:BindingEnergy>
  </phase:BindingEnergyCollection>
"""
        actual_element = solvationenergyextraction.create_binding_energy_collection_element()
        xpath_expression = solvationenergyextraction.generate_binding_energy_xpath_expression("water")
        binding_elements = solvationenergyextraction.get_energy_elements(self.binding_energy_xml,
                                                                        xpath_expression)
        solvationenergyextraction.append_energies_to_collection(actual_element, binding_elements)
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_element, pretty_print=True))
    def test_create_energy_values_element(self):
        """Test to see if expected element is produced.
        """
        expected_xml = """<phase:EnergyValues xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema"/>
"""
        actual_element = solvationenergyextraction.create_energy_values_element()
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_element, pretty_print=True))
    def test_create_free_energy_collection_element(self):
        """Test to see if expected element is produced.
        """
        expected_xml = """<phase:FreeEnergyCollection xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema"/>
"""
        actual_element = solvationenergyextraction.create_free_energy_collection_element()
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_element, pretty_print=True))
    def test_create_binding_energy_collection_element(self):
        """Test to see if expected element is produced.
        """
        expected_xml = """<phase:BindingEnergyCollection xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema"/>
"""
        actual_element = solvationenergyextraction.create_binding_energy_collection_element()
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_element, pretty_print=True))
    def test_get_energy_elements(self):
        """Test to see if expected energy elements are returned.
        """
        expected_xml = """<phase:BindingEnergy xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema" phase:moleculeID="-14.100solute" phase:fromSolventID="" phase:toSolventID="water" phase:valueType="MOLEFRACTION">
            <phase:TotalEnergy>-34.8195308084233</phase:TotalEnergy>
            <phase:EnergyContributions>
                <phase:EnergyContribution phase:ssipID="1">-34.8195308084233</phase:EnergyContribution>
            </phase:EnergyContributions>
        </phase:BindingEnergy>
  
"""
        xpath_expression = solvationenergyextraction.generate_binding_energy_xpath_expression("water")
        actual_elements = solvationenergyextraction.get_energy_elements(self.binding_energy_xml,
                                                                        xpath_expression)
        self.assertEqual(1, len(actual_elements))
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_elements[0], pretty_print=True))
    def test_generate_free_energy_xpath_expression(self):
        """Test to see if expected XPath expression created.
        """
        expected_xpath_expression = "/phase:EnergyValues/phase:FreeEnergyCollection/phase:FreeEnergy[@phase:fromSolventID=''][@phase:toSolventID='water']"
        actual_xpath_expression = solvationenergyextraction.generate_free_energy_xpath_expression("water")
        self.assertEqual(expected_xpath_expression, actual_xpath_expression)
    def test_generate_binding_energy_xpath_expression(self):
        """Test to see if expected XPath expression created.
        """
        expected_xpath_expression = "/phase:EnergyValues/phase:BindingEnergyCollection/phase:BindingEnergy[@phase:fromSolventID=''][@phase:toSolventID='water']"
        actual_xpath_expression = solvationenergyextraction.generate_binding_energy_xpath_expression("water")
        self.assertEqual(expected_xpath_expression, actual_xpath_expression)
    def test_read_xml_file(self):
        """Test to see if expected file is read.
        """
        expected_xml = """<phase:EnergyValues xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema">
  <phase:BindingEnergyCollection><phase:BindingEnergy phase:moleculeID="-14.100solute" phase:fromSolventID="" phase:toSolventID="water" phase:valueType="MOLEFRACTION">
            <phase:TotalEnergy>-34.8195308084233</phase:TotalEnergy>
            <phase:EnergyContributions>
                <phase:EnergyContribution phase:ssipID="1">-34.8195308084233</phase:EnergyContribution>
            </phase:EnergyContributions>
        </phase:BindingEnergy>
  </phase:BindingEnergyCollection>
</phase:EnergyValues>
"""
        self.assertMultiLineEqual(expected_xml, etree.tounicode(self.binding_energy_xml, pretty_print=True))