#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script containing tests for the solventxmlreader.

@author: mark
"""

import logging
import unittest
from lxml import etree
import solventmapcreator.io.solventxmlreader as solventxmlreader

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

class SolventXMLReaderTestCase(unittest.TestCase):
    """Test case for solventxmlreader script tests.
    """
    def setUp(self):
        """Set up for tests.
        """
        self.solvent_list_xml = solventxmlreader.read_element_tree("resources/watersolvent.xml")
        self.water_solvent_element = self.solvent_list_xml.xpath("phase:Solvents/phase:Solvent", namespaces=solventxmlreader.PHASE_NAMESPACE_DICT)[0]
        self.water_molecule_element = self.water_solvent_element.xpath("phase:Molecules/phase:Molecule", namespaces=solventxmlreader.PHASE_NAMESPACE_DICT)[0]
        self.maxDiff=None
    def tearDown(self):
        """Tear down after tests.
        """
        del self.solvent_list_xml
        del self.water_solvent_element
        del self.water_molecule_element
    def test_get_fractional_occupancy_for_solvent_from_file(self):
        """Test to see if expected value is returned.
        """
        expected_value = 55.35 * 4.0 /300.0
        actual_value = solventxmlreader.get_fractional_occupancy_for_solvent_from_file("resources/watersolvent.xml", "water")
        self.assertAlmostEqual(expected_value, actual_value)
    def test_get_fractional_occupancy_for_solvent(self):
        """Test to see if expected fractional occupancy is returned.
        """
        expected_value = 55.35 * 4.0 /300.0
        actual_value = solventxmlreader.get_fractional_occupancy_for_solvent(self.solvent_list_xml, "water")
        self.assertAlmostEqual(expected_value, actual_value)
    def test_calculate_fractional_occupancy_for_solvent(self):
        """Test to see if expected value is returned for the fractional
        occupancy.
        """
        expected_value = 55.35 * 4.0 /300.0
        actual_value = solventxmlreader.calculate_fractional_occupancy_for_solvent(self.water_solvent_element)
        self.assertAlmostEqual(expected_value, actual_value)
    def test_calculate_fractional_occupancy_contribution(self):
        """Test to see if expected contribution to fractional occupancy is returned.
        """
        expected_value = 55.35* 4.0 / 300.0
        actual_value = solventxmlreader.calculate_fractional_occupancy_contribution(self.water_molecule_element)
        self.assertAlmostEqual(expected_value, actual_value)
    def test_get_concentration_value_sum(self):
        """Test to see if expected concentration is returned
        """
        expected_value = 55.3500 * 4
        actual_value = solventxmlreader.get_concentration_value_sum(self.water_molecule_element)
        self.assertAlmostEqual(expected_value, actual_value)
    def test_get_number_of_ssips_in_molecule(self):
        """Test to see if expected number of SSIPs is returned.
        """
        expected_value = 4
        actual_value = solventxmlreader.get_number_of_ssips_in_molecule(self.water_molecule_element)
        self.assertEqual(expected_value, actual_value)
    def test_get_solvent_element_by_id(self):
        """Test to see if expected solvent element is returned.
        """
        expected_xml = '''<phase:Solvent xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema" xmlns:cml="http://www.xml-cml.org/schema" xmlns:ssip="http://www-hunter.ch.cam.ac.uk/SSIP" phase:solventID="water" phase:solventName="water">
      <phase:Molecules>
        <phase:Molecule phase:stdInChIKey="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N"><cml:molecule ssip:stdInChIKey="XLYOFNOQVPJJNP-UHFFFAOYSA-N" cml:id="XLYOFNOQVPJJNP-UHFFFAOYSA-N">
        <cml:atomArray>
            <cml:atom cml:elementType="O" cml:id="a1" cml:x3="0.0569236794019" cml:y3="0.53497425302" cml:z3="-9.99862395488E-5"/>
            <cml:atom cml:elementType="H" cml:id="a2" cml:x3="-0.735721920018" cml:y3="-0.0219270863225" cml:z3="-1.029173866E-4"/>
            <cml:atom cml:elementType="H" cml:id="a3" cml:x3="0.785198240616" cml:y3="-0.103847166698" cml:z3="-1.97096373822E-4"/>
        </cml:atomArray>
        <cml:bondArray>
            <cml:bond cml:atomRefs2="a1 a2" cml:order="1"/>
            <cml:bond cml:atomRefs2="a1 a3" cml:order="1"/>
        </cml:bondArray>
    </cml:molecule>
    <ssip:SurfaceInformation>
        <ssip:TotalSurfaceArea>37.48483410829331</ssip:TotalSurfaceArea>
        <ssip:NegativeSurfaceArea>18.664358683092974</ssip:NegativeSurfaceArea>
        <ssip:PositiveSurfaceArea>18.82047542519977</ssip:PositiveSurfaceArea>
        <ssip:ElectronDensityIsosurface>0.002</ssip:ElectronDensityIsosurface>
        <ssip:NumberOFMEPSPoints>2286</ssip:NumberOFMEPSPoints>
    </ssip:SurfaceInformation>
    <phase:SSIPs phase:units="MOLAR">
        <phase:SSIP ssip:value="2.9387503903833574" ssip:nearestAtomID="a3" cml:x3="1.509794550767402" cml:y3="-0.8784982637871289" cml:z3="-1.6986589692899999E-4" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:units="MOLAR"><phase:TotalConcentration phase:units="MOLAR">55.350000</phase:TotalConcentration></phase:SSIP>
        <phase:SSIP ssip:value="2.9275582734233065" ssip:nearestAtomID="a2" cml:x3="-1.4603207114038939" cml:y3="-0.790973405157027" cml:z3="0.12362427220238399" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:units="MOLAR"><phase:TotalConcentration phase:units="MOLAR">55.350000</phase:TotalConcentration></phase:SSIP>
        <phase:SSIP ssip:value="-6.263074063139137" ssip:nearestAtomID="a1" cml:x3="0.156188781751346" cml:y3="1.6969169177862917" cml:z3="1.172598045109361" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:units="MOLAR"><phase:TotalConcentration phase:units="MOLAR">55.350000</phase:TotalConcentration></phase:SSIP>
        <phase:SSIP ssip:value="-6.607684490829715" ssip:nearestAtomID="a1" cml:x3="0.06855491260794999" cml:y3="1.9788577934582507" cml:z3="-0.77762067563301" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:units="MOLAR"><phase:TotalConcentration phase:units="MOLAR">55.350000</phase:TotalConcentration></phase:SSIP>
    </phase:SSIPs>
</phase:Molecule>
      </phase:Molecules>
    </phase:Solvent>
  
'''
        actual_xml = solventxmlreader.get_solvent_element_by_id(self.solvent_list_xml, "water")
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_xml, pretty_print=True))
    def test_create_solvent_xpath_expression(self):
        """Test to see if expected xpath expression is created.
        """
        expected_expression = 'phase:Solvents/phase:Solvent[@phase:solventID="water"]'
        actual_expression = solventxmlreader.create_solvent_xpath_expression("water")
        self.assertEqual(expected_expression, actual_expression)
    def test_read_element_tree(self):
        """Test to see if expected file is read in.
        """
        expected_xml = '''<phase:SolventList xmlns:cml="http://www.xml-cml.org/schema" xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema" xmlns:ssip="http://www-hunter.ch.cam.ac.uk/SSIP">
  <phase:Solvents>
    <phase:Solvent phase:solventID="water" phase:solventName="water">
      <phase:Molecules>
        <phase:Molecule phase:stdInChIKey="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N"><cml:molecule ssip:stdInChIKey="XLYOFNOQVPJJNP-UHFFFAOYSA-N" cml:id="XLYOFNOQVPJJNP-UHFFFAOYSA-N">
        <cml:atomArray>
            <cml:atom cml:elementType="O" cml:id="a1" cml:x3="0.0569236794019" cml:y3="0.53497425302" cml:z3="-9.99862395488E-5"/>
            <cml:atom cml:elementType="H" cml:id="a2" cml:x3="-0.735721920018" cml:y3="-0.0219270863225" cml:z3="-1.029173866E-4"/>
            <cml:atom cml:elementType="H" cml:id="a3" cml:x3="0.785198240616" cml:y3="-0.103847166698" cml:z3="-1.97096373822E-4"/>
        </cml:atomArray>
        <cml:bondArray>
            <cml:bond cml:atomRefs2="a1 a2" cml:order="1"/>
            <cml:bond cml:atomRefs2="a1 a3" cml:order="1"/>
        </cml:bondArray>
    </cml:molecule>
    <ssip:SurfaceInformation>
        <ssip:TotalSurfaceArea>37.48483410829331</ssip:TotalSurfaceArea>
        <ssip:NegativeSurfaceArea>18.664358683092974</ssip:NegativeSurfaceArea>
        <ssip:PositiveSurfaceArea>18.82047542519977</ssip:PositiveSurfaceArea>
        <ssip:ElectronDensityIsosurface>0.002</ssip:ElectronDensityIsosurface>
        <ssip:NumberOFMEPSPoints>2286</ssip:NumberOFMEPSPoints>
    </ssip:SurfaceInformation>
    <phase:SSIPs phase:units="MOLAR">
        <phase:SSIP ssip:value="2.9387503903833574" ssip:nearestAtomID="a3" cml:x3="1.509794550767402" cml:y3="-0.8784982637871289" cml:z3="-1.6986589692899999E-4" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:units="MOLAR"><phase:TotalConcentration phase:units="MOLAR">55.350000</phase:TotalConcentration></phase:SSIP>
        <phase:SSIP ssip:value="2.9275582734233065" ssip:nearestAtomID="a2" cml:x3="-1.4603207114038939" cml:y3="-0.790973405157027" cml:z3="0.12362427220238399" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:units="MOLAR"><phase:TotalConcentration phase:units="MOLAR">55.350000</phase:TotalConcentration></phase:SSIP>
        <phase:SSIP ssip:value="-6.263074063139137" ssip:nearestAtomID="a1" cml:x3="0.156188781751346" cml:y3="1.6969169177862917" cml:z3="1.172598045109361" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:units="MOLAR"><phase:TotalConcentration phase:units="MOLAR">55.350000</phase:TotalConcentration></phase:SSIP>
        <phase:SSIP ssip:value="-6.607684490829715" ssip:nearestAtomID="a1" cml:x3="0.06855491260794999" cml:y3="1.9788577934582507" cml:z3="-0.77762067563301" phase:moleculeID="XLYOFNOQVPJJNP-UHFFFAOYSA-N" phase:units="MOLAR"><phase:TotalConcentration phase:units="MOLAR">55.350000</phase:TotalConcentration></phase:SSIP>
    </phase:SSIPs>
</phase:Molecule>
      </phase:Molecules>
    </phase:Solvent>
  </phase:Solvents>
</phase:SolventList>
'''
        actual_xml = solventxmlreader.read_element_tree("resources/watersolvent.xml")
        self.assertMultiLineEqual(expected_xml, etree.tounicode(actual_xml, pretty_print=True))
